﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using Phidget22;
using Phidget22.Events;
using Phidget22.ExampleUtils;

namespace PressureSensor_Example {
	public partial class Form1 : Form {
		CommandLineOpen openArgs;
		PressureSensor openCh = null;
		formCleaner cleaner;
		PressureSensor baro = null;
		private ErrorEventBox errorBox;
		private DataGraphBox pressureGraphBox;

		public Form1() {
			InitializeComponent();
			openArgs = new CommandLineOpen(this);
			cleaner = new formCleaner(this);
		}

		public Form1(PressureSensor ch) {
			InitializeComponent();
			openCh = ch;
			cleaner = new formCleaner(this);
		}

		private void Form1_Load(object sender, EventArgs e) {

			errorBox = new ErrorEventBox(this);
			baro = openCh ?? openArgs.makeChannel<PressureSensor>();
			pressureGraphBox = new DataGraphBox(this, "Pressure");

			this.Size = this.MinimumSize;

			baro.Attach += baro_attach;
			baro.Detach += baro_detach;
			baro.Error += baro_error;
			baro.PressureChange += baro_change;
			baro.PropertyChange += baro_propertychange;

			try {
				baro.Open();
			} catch (PhidgetException ex) { errorBox.addMessage("Error opening device: " + ex.Message); }
		}

		private void Form1_FormClosing(object sender, FormClosingEventArgs e) {

			baro.Attach -= baro_attach;
			baro.Detach -= baro_detach;
			baro.Error -= baro_error;
			baro.PressureChange -= baro_change;
			baro.PropertyChange -= baro_propertychange;
			baro.Close();
		}

		void baro_attach(object sender, Phidget22.Events.AttachEventArgs e) {
			phidgetInfoBox1.FillPhidgetInfo((Phidget)sender);

			PressureSensor attachedDevice = (PressureSensor)sender;

			pressureGraphBox.SetTitle(DataGraphBox.GenerateTitle("Pressure", (Phidget)attachedDevice));

			try {
				dataIntervalTrk.Unit = "ms";
				dataIntervalTrk.initializeRange(attachedDevice.MinDataInterval, attachedDevice.MaxDataInterval);
				dataIntervalTrk.trkText = dataIntervalTrk.minTxt = attachedDevice.MinDataInterval.ToString();
				dataIntervalTrk.Minimum = attachedDevice.MinDataInterval;
				dataIntervalTrk.maxTxt = attachedDevice.MaxDataInterval.ToString();
				dataIntervalTrk.Maximum = attachedDevice.MaxDataInterval;
				dataIntervalTrk.Value = attachedDevice.MinDataInterval;

				changeTriggerTrk.Unit = "kPa";
				changeTriggerTrk.minTxt = attachedDevice.MinPressureChangeTrigger.ToString();
				changeTriggerTrk.Minimum = attachedDevice.MinPressureChangeTrigger;
				changeTriggerTrk.Maximum = attachedDevice.MaxPressureChangeTrigger;
				changeTriggerTrk.maxTxt = attachedDevice.MaxPressureChangeTrigger.ToString();
				changeTriggerTrk.Value = attachedDevice.PressureChangeTrigger;
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error initializing device: " + ex.Message);
			}

			settingsBox.Visible = true;
			outputBox.Visible = true;
		}
		void baro_detach(object sender, Phidget22.Events.DetachEventArgs e) {
			phidgetInfoBox1.Clear();
			cleaner.clean();

			settingsBox.Visible = false;
			outputBox.Visible = false;
		}
		void baro_error(object sender, Phidget22.Events.ErrorEventArgs e) {
			errorBox.addMessage(e.Description);

			if (e.Code == ErrorEventCode.Saturation)
				pressureGraphBox.addGap();
		}

		private void baro_propertychange(object sender, PropertyChangeEventArgs e) {

			switch (e.PropertyName) {
			case "DataInterval":
				if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL) {
					dataIntervalTrk.Value = baro.DataInterval;
					updateGraphDataInterval();
				}
				break;
			case "PressureChangeTrigger":
				changeTriggerTrk.Value = baro.PressureChangeTrigger;
				break;
			case "DataRate":
				if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE) {
					dataIntervalTrk.Value = baro.DataRate;
					updateGraphDataInterval();
				}
				break;
			}
		}

		void baro_change(object sender, Phidget22.Events.PressureSensorPressureChangeEventArgs e) {
			try {
				barovalue.Text = e.Pressure.ToString() + " kPa";
				pressureGraphBox.addData(e.Pressure, baro.DataInterval);
			} catch (PhidgetException ex) { errorBox.addMessage("Error reading pressure: " + ex.Message); }
		}


		private void dataIntervalTrk_Scroll(object sender, EventArgs e) {
			if (MouseButtons != MouseButtons.Left) {
				try {
					baro.DataInterval = (int)dataIntervalTrk.Value;
				} catch (Exception ex) {
					errorBox.addMessage("Error setting data interval: " + ex.Message);
				}
			}
			dataIntervalTrk.TrueValue = dataIntervalTrk.Value;
		}

		private void changeTriggertrk_Scroll(object sender, EventArgs e) {
			if (MouseButtons != MouseButtons.Left) {
				try {
					baro.PressureChangeTrigger = changeTriggerTrk.Value;
				} catch (Exception ex) {
					errorBox.addMessage("Error setting change trigger: " + ex.Message);
				}
			}
		}

		private void changeTriggertrk_MouseUp(object sender, MouseEventArgs e) {
			try {
				baro.PressureChangeTrigger = changeTriggerTrk.Value;
			} catch (Exception ex) {
				errorBox.addMessage("Error setting change trigger: " + ex.Message);
			}
		}

		private void updateGraphDataInterval() {
			try {
				if (baro.PressureChangeTrigger == 0)
					pressureGraphBox.DataInterval = (1000.0 / baro.DataRate);
				else
					pressureGraphBox.DataInterval = null;
			} catch (PhidgetException ex) { errorBox.addMessage("Error updating data interval: " + ex.Message); }
		}
		private void dataIntervalTrk_MouseUp(object sender, MouseEventArgs e) {
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL) {
				try {
					baro.DataInterval = (int)Math.Round(dataIntervalTrk.Value);
					updateGraphDataInterval();
				} catch (Exception ex) {
					errorBox.addMessage("Error setting data interval: " + ex.Message);
				}
			} else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE) {
				try {
					baro.DataRate = dataIntervalTrk.Value;
					updateGraphDataInterval();
				} catch (Exception ex) {
					errorBox.addMessage("Error setting data rate: " + ex.Message);
				}
			}
			dataIntervalTrk.TrueValue = dataIntervalTrk.Value;
		}

		private void pressureGraphButton_Click(object sender, EventArgs e) {
			if (!pressureGraphBox.Visible)
				pressureGraphBox.Show();
		}

		private void dataIntervalTrk_SetClick(object sender, EventArgs e) {
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL) {
				try {
					baro.DataInterval = (int)dataIntervalTrk.Value;
					updateGraphDataInterval();
				} catch (Exception ex) { errorBox.addMessage("Error setting data interval: " + ex.Message); }
			} else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE) {
				try {
					baro.DataRate = dataIntervalTrk.Value;
					updateGraphDataInterval();
				} catch (Exception ex) { errorBox.addMessage("Error setting data rate: " + ex.Message); }
			}

		}

		private void dataIntervalTrk_SetEnter(object sender, KeyEventArgs e) {
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL) {
				if (e.KeyCode == Keys.Enter) {
					try {
						baro.DataInterval = (int)dataIntervalTrk.Value;
						updateGraphDataInterval();
					} catch (Exception ex) { errorBox.addMessage("Error setting data interval: " + ex.Message); }
				}
			} else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE) {
				if (e.KeyCode == Keys.Enter) {
					try {
						baro.DataRate = dataIntervalTrk.Value;
						updateGraphDataInterval();
					} catch (Exception ex) { errorBox.addMessage("Error setting data rate: " + ex.Message); }
				}
			}
		}

		private void changeTriggerTrk_SetClick(object sender, EventArgs e) {
			try {
				baro.PressureChangeTrigger = changeTriggerTrk.Value;
			} catch (Exception ex) { errorBox.addMessage("Error setting change trigger: " + ex.Message); }
		}

		private void changeTriggerTrk_SetEnter(object sender, KeyEventArgs e) {
			if (e.KeyCode == Keys.Enter) {
				try {
					baro.PressureChangeTrigger = changeTriggerTrk.Value;
				} catch (Exception ex) { errorBox.addMessage("Error setting change trigger: " + ex.Message); }
			}
		}

		private void dataIntervalTrk_ModeSwitch(object sender, EventArgs e) {
			dataIntervalTrk.SwitchModes();
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE) {
				try {
					dataIntervalTrk.isIntValue = false;
					dataIntervalTrk.isFixedRange = false;
					dataIntervalTrk.Unit = "Hz";
					dataIntervalTrk.Maximum = baro.MaxDataRate;
					dataIntervalTrk.Minimum = baro.MinDataRate;
					dataIntervalTrk.maxTxt = baro.MaxDataRate.ToString();
					dataIntervalTrk.minTxt = baro.MinDataRate.ToString("F3");
					dataIntervalTrk.trkText = baro.DataRate.ToString() + "Hz";
					dataIntervalTrk.Value = baro.DataRate;
				} catch (Exception ex) { errorBox.addMessage("Error setting data rate: " + ex.Message); }

			} else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL) {
				try {
					dataIntervalTrk.isIntValue = true;
					dataIntervalTrk.isFixedRange = true;
					dataIntervalTrk.Unit = "ms";
					dataIntervalTrk.Maximum = baro.MaxDataInterval;
					dataIntervalTrk.Minimum = baro.MinDataInterval;
					dataIntervalTrk.maxTxt = baro.MaxDataInterval.ToString();
					dataIntervalTrk.minTxt = baro.MinDataInterval.ToString();
					dataIntervalTrk.trkText = baro.DataInterval.ToString() + "ms";
					dataIntervalTrk.Value = baro.DataInterval;
				} catch (Exception ex) { errorBox.addMessage("Error setting data interval: " + ex.Message); }
			}
			dataIntervalTrk.TrueValue = dataIntervalTrk.Value;
		}
	}
}
